/******************************************************************************
*
* Freescale Semiconductor Inc.
* (c) Copyright 2013-2016 Freescale Semiconductor, Inc.
* Copyright 2016-2024 NXP
* ALL RIGHTS RESERVED.
*
****************************************************************************//*!
*
* @file      lin_cfg.h
*
* @author    FPT Software
*
* @version   1.0
*
* @date      Tue Nov 19 16:50:38 GMT+08:00 2024
*
* @brief     Hardware configuration file
*
******************************************************************************/
#ifndef    _LIN_CFG_H_
#define    _LIN_CFG_H_
#include "lin_hw_cfg.h"
/* Define operating mode */
#define _MASTER_MODE_     0
#define _SLAVE_MODE_      1
#define LIN_MODE   _SLAVE_MODE_
/* Define protocol version */
#define PROTOCOL_21       0
#define PROTOCOL_J2602    1
#define PROTOCOL_20       2
#define LIN_PROTOCOL    PROTOCOL_21

#define SCI_ADDR        SCI0_ADDR    /* For slave */

#define LIN_BAUD_RATE    19200    	 /*For slave*/
/**********************************************************************/
/***************          Diagnostic class selection  *****************/
/**********************************************************************/
#define _DIAG_CLASS_I_          0
#define _DIAG_CLASS_II_         1
#define _DIAG_CLASS_III_        2

#define _DIAG_CLASS_SUPPORT_    _DIAG_CLASS_III_

#define MAX_LENGTH_SERVICE 64

#define MAX_QUEUE_SIZE 11


#define _DIAG_NUMBER_OF_SERVICES_    14

#define DIAGSRV_READ_BY_IDENTIFIER_ORDER    0

#define DIAGSRV_SAVE_CONFIGURATION_ORDER    1

#define DIAGSRV_ASSIGN_FRAME_ID_RANGE_ORDER    2

#define DIAGSRV_READ_DATA_BY_IDENTIFIER_ORDER    3

#define DIAGSRV_SESSION_CONTROL_ORDER    4

#define DIAGSRV_IO_CONTROL_BY_IDENTIFIER_ORDER    5

#define DIAGSRV_FAULT_MEMORY_READ_ORDER    6

#define DIAGSRV_FAULT_MEMORY_CLEAR_ORDER    7

#define DIAGSRV_WRITE_DATA_BY_IDENTIFIER_ORDER    8

#define DIAGSRV_ROUTINECONTROL_ORDER    9

#define DIAGSRV_COMMUNICATIONCONTROL_ORDER    10

#define DIAGSRV_ECURESET_ORDER    11

#define DIAGSRV_TESTERPRESENT_ORDER    12

#define DIAGSRV_ASSIGN_FRAME_IDENTIFIER_ORDER    13


/**************** FRAME SUPPORT DEFINITION ******************/
#define _TL_SINGLE_FRAME_       0
#define _TL_MULTI_FRAME_        1

#define _TL_FRAME_SUPPORT_      _TL_MULTI_FRAME_

/* frame buffer size */
#define LIN_FRAME_BUF_SIZE			30
#define LIN_FLAG_BUF_SIZE			6

/**********************************************************************/
/***************               Interfaces           *******************/
/**********************************************************************/
typedef enum {
   LI0
}l_ifc_handle;

/**********************************************************************/
/***************               Signals              *******************/
/**********************************************************************/
/* Number of signals */
#define LIN_NUM_OF_SIGS  37
/* List of signals */
typedef enum {

   /* Interface_name = LI0 */

   LI0_MFL_Zaehler

   , LI0_MFL_Toggle
  
   , LI0_MFL_Lokalaktiv
  
   , LI0_MFL_M_Taste
  
   , LI0_void
  
   , LI0_MFL_Tastencode_1
  
   , LI0_MFL_Tastencode_2
  
   , LI0_MFL_Eventcode_1
  
   , LI0_MFL_Eventcode_2
  
   , LI0_MFL_Marke
  
   , LI0_MFL_Variante
  
   , LI0_MFL_Dummy_0_Signal_1
  
   , LI0_MFL_Tip_Down
  
   , LI0_MFL_Tip_Up
  
   , LI0_MFL_SatModul_links_Err
  
   , LI0_MFL_SatModul_rechts_Err
  
   , LI0_MFL_Dummy_0_Signal_2
  
   , LI0_MFL_Dummy_0_Signal_3
  
   , LI0_MFL_LR_HZG_Status
  
   , LI0_MFL_LR_HZG_Err
  
   , LI0_MFL_Signalhorn
  
   , LI0_MFL_Signalhorn_Err
  
   , LI0_MFL_Tip_links_Err
  
   , LI0_MFL_Tip_rechts_Err
  
   , LI0_MFL_Taste_links_Err
  
   , LI0_MFL_Taste_rechts_Err
  
   , LI0_MFL_ECU_Err
  
   , LI0_MFL_Response_Err
  
   , LI0_DI_KL_58xs
  
   , LI0_DI_Emblem
  
   , LI0_Klemme_15_MFL
  
   , LI0_void_1
  
   , LI0_Dimmung_01_gueltig
  
   , LI0_DI_KL_58xt
  
   , LI0_void_2
  
   , LI0_void_3
  
   , LI0_P_Mode_MFL
  
} l_signal_handle;
/**********************************************************************/
/*****************               Frame             ********************/
/**********************************************************************/
/* Number of frames */
#define LIN_NUM_OF_FRMS  4
/* List of frames */
typedef enum {
/* All frames for master node */

   /* Interface_name = LI0 */

   LI0_MFLs_01

   , LI0_MFLe_01
  
   , LI0_MasterReq
  
   , LI0_SlaveResp
  
} l_frame_handle;
/**********************************************************************/
/***************             Configuration          *******************/
/**********************************************************************/
/* Size of configuration in ROM and RAM used for interface: LI1 */
#define LIN_SIZE_OF_CFG  6
#define LIN_CFG_FRAME_NUM  2
/*********************************************************************
 * global macros
 *********************************************************************/
#define l_bool_rd(SIGNAL) l_bool_rd_##SIGNAL()
#define l_bool_wr(SIGNAL, A) l_bool_wr_##SIGNAL(A)
#define l_u8_rd(SIGNAL) l_u8_rd_##SIGNAL()
#define l_u8_wr(SIGNAL, A) l_u8_wr_##SIGNAL(A)
#define l_u16_rd(SIGNAL) l_u16_rd_##SIGNAL()
#define l_u16_wr(SIGNAL, A) l_u16_wr_##SIGNAL(A)
#define l_bytes_rd(SIGNAL, start, count, data)  l_bytes_rd_##SIGNAL(start, count, data)
#define l_bytes_wr(SIGNAL, start, count, data) l_bytes_wr_##SIGNAL(start, count, data)
#define l_flg_tst(FLAG) l_flg_tst_##FLAG()
#define l_flg_clr(FLAG) l_flg_clr_##FLAG()
#define LIN_TEST_BIT(A,B) ((l_bool)((((A) & (1U << (B))) != 0U) ? 1U : 0U))
#define LIN_SET_BIT(A,B)                      ((A) |= (l_u8) (1U << (B)))
#define LIN_CLEAR_BIT(A,B)               ((A) &= ((l_u8) (~(1U << (B)))))
#define LIN_BYTE_MASK  ((l_u16)(((l_u16)((l_u16)1 << CHAR_BIT)) - (l_u16)1))
#define LIN_FRAME_LEN_MAX                                             10U

/* Returns the low byte of the 32-bit value    */
#define BYTE_0(n)                              ((l_u8)((n) & (l_u8)0xFF))
/* Returns the second byte of the 32-bit value */
#define BYTE_1(n)                        ((l_u8)(BYTE_0((n) >> (l_u8)8)))
/* Returns the third byte of the 32-bit value  */
#define BYTE_2(n)                       ((l_u8)(BYTE_0((n) >> (l_u8)16)))
/* Returns high byte of the 32-bit value       */
#define BYTE_3(n)                       ((l_u8)(BYTE_0((n) >> (l_u8)24)))

/*
 * defines for signal access
 */



#define LIN_BYTE_OFFSET_LI0_MFL_Zaehler    0U
#define LIN_BIT_OFFSET_LI0_MFL_Zaehler    0U
#define LIN_SIGNAL_SIZE_LI0_MFL_Zaehler    4U
#define LIN_FLAG_BYTE_OFFSET_LI0_MFL_Zaehler    0U
#define LIN_FLAG_BIT_OFFSET_LI0_MFL_Zaehler    0U


#define LIN_BYTE_OFFSET_LI0_MFL_Toggle    0U
#define LIN_BIT_OFFSET_LI0_MFL_Toggle    4U
#define LIN_SIGNAL_SIZE_LI0_MFL_Toggle    1U
#define LIN_FLAG_BYTE_OFFSET_LI0_MFL_Toggle    0U
#define LIN_FLAG_BIT_OFFSET_LI0_MFL_Toggle    1U


#define LIN_BYTE_OFFSET_LI0_MFL_Lokalaktiv    0U
#define LIN_BIT_OFFSET_LI0_MFL_Lokalaktiv    5U
#define LIN_SIGNAL_SIZE_LI0_MFL_Lokalaktiv    1U
#define LIN_FLAG_BYTE_OFFSET_LI0_MFL_Lokalaktiv    0U
#define LIN_FLAG_BIT_OFFSET_LI0_MFL_Lokalaktiv    2U


#define LIN_BYTE_OFFSET_LI0_MFL_M_Taste    0U
#define LIN_BIT_OFFSET_LI0_MFL_M_Taste    6U
#define LIN_SIGNAL_SIZE_LI0_MFL_M_Taste    1U
#define LIN_FLAG_BYTE_OFFSET_LI0_MFL_M_Taste    0U
#define LIN_FLAG_BIT_OFFSET_LI0_MFL_M_Taste    3U


#define LIN_BYTE_OFFSET_LI0_void    0U
#define LIN_BIT_OFFSET_LI0_void    7U
#define LIN_SIGNAL_SIZE_LI0_void    1U
#define LIN_FLAG_BYTE_OFFSET_LI0_void    0U
#define LIN_FLAG_BIT_OFFSET_LI0_void    4U


#define LIN_BYTE_OFFSET_LI0_MFL_Tastencode_1    1U
#define LIN_BIT_OFFSET_LI0_MFL_Tastencode_1    0U
#define LIN_SIGNAL_SIZE_LI0_MFL_Tastencode_1    8U
#define LIN_FLAG_BYTE_OFFSET_LI0_MFL_Tastencode_1    0U
#define LIN_FLAG_BIT_OFFSET_LI0_MFL_Tastencode_1    5U


#define LIN_BYTE_OFFSET_LI0_MFL_Tastencode_2    2U
#define LIN_BIT_OFFSET_LI0_MFL_Tastencode_2    0U
#define LIN_SIGNAL_SIZE_LI0_MFL_Tastencode_2    8U
#define LIN_FLAG_BYTE_OFFSET_LI0_MFL_Tastencode_2    0U
#define LIN_FLAG_BIT_OFFSET_LI0_MFL_Tastencode_2    6U


#define LIN_BYTE_OFFSET_LI0_MFL_Eventcode_1    3U
#define LIN_BIT_OFFSET_LI0_MFL_Eventcode_1    0U
#define LIN_SIGNAL_SIZE_LI0_MFL_Eventcode_1    4U
#define LIN_FLAG_BYTE_OFFSET_LI0_MFL_Eventcode_1    0U
#define LIN_FLAG_BIT_OFFSET_LI0_MFL_Eventcode_1    7U


#define LIN_BYTE_OFFSET_LI0_MFL_Eventcode_2    3U
#define LIN_BIT_OFFSET_LI0_MFL_Eventcode_2    4U
#define LIN_SIGNAL_SIZE_LI0_MFL_Eventcode_2    4U
#define LIN_FLAG_BYTE_OFFSET_LI0_MFL_Eventcode_2    1U
#define LIN_FLAG_BIT_OFFSET_LI0_MFL_Eventcode_2    0U


#define LIN_BYTE_OFFSET_LI0_MFL_Marke    4U
#define LIN_BIT_OFFSET_LI0_MFL_Marke    0U
#define LIN_SIGNAL_SIZE_LI0_MFL_Marke    4U
#define LIN_FLAG_BYTE_OFFSET_LI0_MFL_Marke    1U
#define LIN_FLAG_BIT_OFFSET_LI0_MFL_Marke    1U


#define LIN_BYTE_OFFSET_LI0_MFL_Variante    4U
#define LIN_BIT_OFFSET_LI0_MFL_Variante    4U
#define LIN_SIGNAL_SIZE_LI0_MFL_Variante    4U
#define LIN_FLAG_BYTE_OFFSET_LI0_MFL_Variante    1U
#define LIN_FLAG_BIT_OFFSET_LI0_MFL_Variante    2U


#define LIN_BYTE_OFFSET_LI0_MFL_Dummy_0_Signal_1    5U
#define LIN_BIT_OFFSET_LI0_MFL_Dummy_0_Signal_1    0U
#define LIN_SIGNAL_SIZE_LI0_MFL_Dummy_0_Signal_1    8U
#define LIN_FLAG_BYTE_OFFSET_LI0_MFL_Dummy_0_Signal_1    1U
#define LIN_FLAG_BIT_OFFSET_LI0_MFL_Dummy_0_Signal_1    3U


#define LIN_BYTE_OFFSET_LI0_MFL_Tip_Down    6U
#define LIN_BIT_OFFSET_LI0_MFL_Tip_Down    0U
#define LIN_SIGNAL_SIZE_LI0_MFL_Tip_Down    1U
#define LIN_FLAG_BYTE_OFFSET_LI0_MFL_Tip_Down    1U
#define LIN_FLAG_BIT_OFFSET_LI0_MFL_Tip_Down    4U


#define LIN_BYTE_OFFSET_LI0_MFL_Tip_Up    6U
#define LIN_BIT_OFFSET_LI0_MFL_Tip_Up    1U
#define LIN_SIGNAL_SIZE_LI0_MFL_Tip_Up    1U
#define LIN_FLAG_BYTE_OFFSET_LI0_MFL_Tip_Up    1U
#define LIN_FLAG_BIT_OFFSET_LI0_MFL_Tip_Up    5U


#define LIN_BYTE_OFFSET_LI0_MFL_SatModul_links_Err    6U
#define LIN_BIT_OFFSET_LI0_MFL_SatModul_links_Err    2U
#define LIN_SIGNAL_SIZE_LI0_MFL_SatModul_links_Err    1U
#define LIN_FLAG_BYTE_OFFSET_LI0_MFL_SatModul_links_Err    1U
#define LIN_FLAG_BIT_OFFSET_LI0_MFL_SatModul_links_Err    6U


#define LIN_BYTE_OFFSET_LI0_MFL_SatModul_rechts_Err    6U
#define LIN_BIT_OFFSET_LI0_MFL_SatModul_rechts_Err    3U
#define LIN_SIGNAL_SIZE_LI0_MFL_SatModul_rechts_Err    1U
#define LIN_FLAG_BYTE_OFFSET_LI0_MFL_SatModul_rechts_Err    1U
#define LIN_FLAG_BIT_OFFSET_LI0_MFL_SatModul_rechts_Err    7U


#define LIN_BYTE_OFFSET_LI0_MFL_Dummy_0_Signal_2    6U
#define LIN_BIT_OFFSET_LI0_MFL_Dummy_0_Signal_2    4U
#define LIN_SIGNAL_SIZE_LI0_MFL_Dummy_0_Signal_2    1U
#define LIN_FLAG_BYTE_OFFSET_LI0_MFL_Dummy_0_Signal_2    2U
#define LIN_FLAG_BIT_OFFSET_LI0_MFL_Dummy_0_Signal_2    0U


#define LIN_BYTE_OFFSET_LI0_MFL_Dummy_0_Signal_3    6U
#define LIN_BIT_OFFSET_LI0_MFL_Dummy_0_Signal_3    5U
#define LIN_SIGNAL_SIZE_LI0_MFL_Dummy_0_Signal_3    1U
#define LIN_FLAG_BYTE_OFFSET_LI0_MFL_Dummy_0_Signal_3    2U
#define LIN_FLAG_BIT_OFFSET_LI0_MFL_Dummy_0_Signal_3    1U


#define LIN_BYTE_OFFSET_LI0_MFL_LR_HZG_Status    6U
#define LIN_BIT_OFFSET_LI0_MFL_LR_HZG_Status    6U
#define LIN_SIGNAL_SIZE_LI0_MFL_LR_HZG_Status    1U
#define LIN_FLAG_BYTE_OFFSET_LI0_MFL_LR_HZG_Status    2U
#define LIN_FLAG_BIT_OFFSET_LI0_MFL_LR_HZG_Status    2U


#define LIN_BYTE_OFFSET_LI0_MFL_LR_HZG_Err    6U
#define LIN_BIT_OFFSET_LI0_MFL_LR_HZG_Err    7U
#define LIN_SIGNAL_SIZE_LI0_MFL_LR_HZG_Err    1U
#define LIN_FLAG_BYTE_OFFSET_LI0_MFL_LR_HZG_Err    2U
#define LIN_FLAG_BIT_OFFSET_LI0_MFL_LR_HZG_Err    3U


#define LIN_BYTE_OFFSET_LI0_MFL_Signalhorn    7U
#define LIN_BIT_OFFSET_LI0_MFL_Signalhorn    0U
#define LIN_SIGNAL_SIZE_LI0_MFL_Signalhorn    1U
#define LIN_FLAG_BYTE_OFFSET_LI0_MFL_Signalhorn    2U
#define LIN_FLAG_BIT_OFFSET_LI0_MFL_Signalhorn    4U


#define LIN_BYTE_OFFSET_LI0_MFL_Signalhorn_Err    7U
#define LIN_BIT_OFFSET_LI0_MFL_Signalhorn_Err    1U
#define LIN_SIGNAL_SIZE_LI0_MFL_Signalhorn_Err    1U
#define LIN_FLAG_BYTE_OFFSET_LI0_MFL_Signalhorn_Err    2U
#define LIN_FLAG_BIT_OFFSET_LI0_MFL_Signalhorn_Err    5U


#define LIN_BYTE_OFFSET_LI0_MFL_Tip_links_Err    7U
#define LIN_BIT_OFFSET_LI0_MFL_Tip_links_Err    2U
#define LIN_SIGNAL_SIZE_LI0_MFL_Tip_links_Err    1U
#define LIN_FLAG_BYTE_OFFSET_LI0_MFL_Tip_links_Err    2U
#define LIN_FLAG_BIT_OFFSET_LI0_MFL_Tip_links_Err    6U


#define LIN_BYTE_OFFSET_LI0_MFL_Tip_rechts_Err    7U
#define LIN_BIT_OFFSET_LI0_MFL_Tip_rechts_Err    3U
#define LIN_SIGNAL_SIZE_LI0_MFL_Tip_rechts_Err    1U
#define LIN_FLAG_BYTE_OFFSET_LI0_MFL_Tip_rechts_Err    2U
#define LIN_FLAG_BIT_OFFSET_LI0_MFL_Tip_rechts_Err    7U


#define LIN_BYTE_OFFSET_LI0_MFL_Taste_links_Err    7U
#define LIN_BIT_OFFSET_LI0_MFL_Taste_links_Err    4U
#define LIN_SIGNAL_SIZE_LI0_MFL_Taste_links_Err    1U
#define LIN_FLAG_BYTE_OFFSET_LI0_MFL_Taste_links_Err    3U
#define LIN_FLAG_BIT_OFFSET_LI0_MFL_Taste_links_Err    0U


#define LIN_BYTE_OFFSET_LI0_MFL_Taste_rechts_Err    7U
#define LIN_BIT_OFFSET_LI0_MFL_Taste_rechts_Err    5U
#define LIN_SIGNAL_SIZE_LI0_MFL_Taste_rechts_Err    1U
#define LIN_FLAG_BYTE_OFFSET_LI0_MFL_Taste_rechts_Err    3U
#define LIN_FLAG_BIT_OFFSET_LI0_MFL_Taste_rechts_Err    1U


#define LIN_BYTE_OFFSET_LI0_MFL_ECU_Err    7U
#define LIN_BIT_OFFSET_LI0_MFL_ECU_Err    6U
#define LIN_SIGNAL_SIZE_LI0_MFL_ECU_Err    1U
#define LIN_FLAG_BYTE_OFFSET_LI0_MFL_ECU_Err    3U
#define LIN_FLAG_BIT_OFFSET_LI0_MFL_ECU_Err    2U


#define LIN_BYTE_OFFSET_LI0_MFL_Response_Err    7U
#define LIN_BIT_OFFSET_LI0_MFL_Response_Err    7U
#define LIN_SIGNAL_SIZE_LI0_MFL_Response_Err    1U
#define LIN_FLAG_BYTE_OFFSET_LI0_MFL_Response_Err    3U
#define LIN_FLAG_BIT_OFFSET_LI0_MFL_Response_Err    3U


#define LIN_BYTE_OFFSET_LI0_DI_KL_58xs    8U
#define LIN_BIT_OFFSET_LI0_DI_KL_58xs    0U
#define LIN_SIGNAL_SIZE_LI0_DI_KL_58xs    7U
#define LIN_FLAG_BYTE_OFFSET_LI0_DI_KL_58xs    4U
#define LIN_FLAG_BIT_OFFSET_LI0_DI_KL_58xs    0U


#define LIN_BYTE_OFFSET_LI0_DI_Emblem    8U
#define LIN_BIT_OFFSET_LI0_DI_Emblem    7U
#define LIN_SIGNAL_SIZE_LI0_DI_Emblem    1U
#define LIN_FLAG_BYTE_OFFSET_LI0_DI_Emblem    4U
#define LIN_FLAG_BIT_OFFSET_LI0_DI_Emblem    1U


#define LIN_BYTE_OFFSET_LI0_Klemme_15_MFL    9U
#define LIN_BIT_OFFSET_LI0_Klemme_15_MFL    0U
#define LIN_SIGNAL_SIZE_LI0_Klemme_15_MFL    1U
#define LIN_FLAG_BYTE_OFFSET_LI0_Klemme_15_MFL    4U
#define LIN_FLAG_BIT_OFFSET_LI0_Klemme_15_MFL    2U


#define LIN_BYTE_OFFSET_LI0_void_1    9U
#define LIN_BIT_OFFSET_LI0_void_1    1U
#define LIN_SIGNAL_SIZE_LI0_void_1    6U
#define LIN_FLAG_BYTE_OFFSET_LI0_void_1    4U
#define LIN_FLAG_BIT_OFFSET_LI0_void_1    3U


#define LIN_BYTE_OFFSET_LI0_Dimmung_01_gueltig    9U
#define LIN_BIT_OFFSET_LI0_Dimmung_01_gueltig    7U
#define LIN_SIGNAL_SIZE_LI0_Dimmung_01_gueltig    1U
#define LIN_FLAG_BYTE_OFFSET_LI0_Dimmung_01_gueltig    4U
#define LIN_FLAG_BIT_OFFSET_LI0_Dimmung_01_gueltig    4U


#define LIN_BYTE_OFFSET_LI0_DI_KL_58xt    10U
#define LIN_BIT_OFFSET_LI0_DI_KL_58xt    0U
#define LIN_SIGNAL_SIZE_LI0_DI_KL_58xt    7U
#define LIN_FLAG_BYTE_OFFSET_LI0_DI_KL_58xt    4U
#define LIN_FLAG_BIT_OFFSET_LI0_DI_KL_58xt    5U


#define LIN_BYTE_OFFSET_LI0_void_2    10U
#define LIN_BIT_OFFSET_LI0_void_2    7U
#define LIN_SIGNAL_SIZE_LI0_void_2    1U
#define LIN_FLAG_BYTE_OFFSET_LI0_void_2    4U
#define LIN_FLAG_BIT_OFFSET_LI0_void_2    6U


#define LIN_BYTE_OFFSET_LI0_void_3    11U
#define LIN_BIT_OFFSET_LI0_void_3    0U
#define LIN_SIGNAL_SIZE_LI0_void_3    7U
#define LIN_FLAG_BYTE_OFFSET_LI0_void_3    4U
#define LIN_FLAG_BIT_OFFSET_LI0_void_3    7U


#define LIN_BYTE_OFFSET_LI0_P_Mode_MFL    11U
#define LIN_BIT_OFFSET_LI0_P_Mode_MFL    7U
#define LIN_SIGNAL_SIZE_LI0_P_Mode_MFL    1U
#define LIN_FLAG_BYTE_OFFSET_LI0_P_Mode_MFL    5U
#define LIN_FLAG_BIT_OFFSET_LI0_P_Mode_MFL    0U




#define LIN_FLAG_BYTE_OFFSET_LI0_MFLs_01             0
#define LIN_FLAG_BIT_OFFSET_LI0_MFLs_01              0

#define LIN_FLAG_BYTE_OFFSET_LI0_MFLe_01             4
#define LIN_FLAG_BIT_OFFSET_LI0_MFLe_01              0


/**********************************************************************/
/***************        Static API Functions        *******************/
/**********************************************************************/
/*
 * the static signal access macros
 */


 
/* static access macros for signal LI0_MFL_Zaehler */
 
#define l_u8_rd_LI0_MFL_Zaehler() \
    ((l_u8)  (((lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Zaehler]) >> 0U) & 0x0fU))


#define l_u8_wr_LI0_MFL_Zaehler(A) \
    { \
    lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Zaehler] = \
    (l_u8)((lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Zaehler] & 0xf0U) | \
    (((A) << 0U) & 0x0fU)); \
    LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Zaehler],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Zaehler); \
    }


/* static access macros for signal LI0_MFL_Toggle */

 
#define l_bool_rd_LI0_MFL_Toggle() \
    (LIN_TEST_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Toggle], \
    LIN_BIT_OFFSET_LI0_MFL_Toggle))

#define l_bool_wr_LI0_MFL_Toggle(A) \
    {(A) ? \
    (LIN_SET_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Toggle], \
    LIN_BIT_OFFSET_LI0_MFL_Toggle)):\
    (LIN_CLEAR_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Toggle], \
    LIN_BIT_OFFSET_LI0_MFL_Toggle));\
    LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Toggle],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Toggle);}
/* static access macros for signal LI0_MFL_Lokalaktiv */

 
#define l_bool_rd_LI0_MFL_Lokalaktiv() \
    (LIN_TEST_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Lokalaktiv], \
    LIN_BIT_OFFSET_LI0_MFL_Lokalaktiv))

#define l_bool_wr_LI0_MFL_Lokalaktiv(A) \
    {(A) ? \
    (LIN_SET_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Lokalaktiv], \
    LIN_BIT_OFFSET_LI0_MFL_Lokalaktiv)):\
    (LIN_CLEAR_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Lokalaktiv], \
    LIN_BIT_OFFSET_LI0_MFL_Lokalaktiv));\
    LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Lokalaktiv],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Lokalaktiv);}
/* static access macros for signal LI0_MFL_M_Taste */

 
#define l_bool_rd_LI0_MFL_M_Taste() \
    (LIN_TEST_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_M_Taste], \
    LIN_BIT_OFFSET_LI0_MFL_M_Taste))

#define l_bool_wr_LI0_MFL_M_Taste(A) \
    {(A) ? \
    (LIN_SET_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_M_Taste], \
    LIN_BIT_OFFSET_LI0_MFL_M_Taste)):\
    (LIN_CLEAR_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_M_Taste], \
    LIN_BIT_OFFSET_LI0_MFL_M_Taste));\
    LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_M_Taste],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_M_Taste);}
/* static access macros for signal LI0_void */

 
#define l_bool_rd_LI0_void() \
    (LIN_TEST_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_void], \
    LIN_BIT_OFFSET_LI0_void))

#define l_bool_wr_LI0_void(A) \
    {(A) ? \
    (LIN_SET_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_void], \
    LIN_BIT_OFFSET_LI0_void)):\
    (LIN_CLEAR_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_void], \
    LIN_BIT_OFFSET_LI0_void));\
    LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_void],\
         LIN_FLAG_BIT_OFFSET_LI0_void);}
 
/* static access macros for signal LI0_MFL_Tastencode_1 */
 
#define l_u8_rd_LI0_MFL_Tastencode_1() \
    ((l_u8)  (((lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Tastencode_1]) >> 0U) & 0xffU))


#define l_u8_wr_LI0_MFL_Tastencode_1(A) \
    { \
    lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Tastencode_1] = \
    (l_u8)((lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Tastencode_1] & 0x00U) | \
    (((A) << 0U) & 0xffU)); \
    LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Tastencode_1],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Tastencode_1); \
    }


 
/* static access macros for signal LI0_MFL_Tastencode_2 */
 
#define l_u8_rd_LI0_MFL_Tastencode_2() \
    ((l_u8)  (((lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Tastencode_2]) >> 0U) & 0xffU))


#define l_u8_wr_LI0_MFL_Tastencode_2(A) \
    { \
    lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Tastencode_2] = \
    (l_u8)((lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Tastencode_2] & 0x00U) | \
    (((A) << 0U) & 0xffU)); \
    LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Tastencode_2],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Tastencode_2); \
    }


 
/* static access macros for signal LI0_MFL_Eventcode_1 */
 
#define l_u8_rd_LI0_MFL_Eventcode_1() \
    ((l_u8)  (((lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Eventcode_1]) >> 0U) & 0x0fU))


#define l_u8_wr_LI0_MFL_Eventcode_1(A) \
    { \
    lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Eventcode_1] = \
    (l_u8)((lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Eventcode_1] & 0xf0U) | \
    (((A) << 0U) & 0x0fU)); \
    LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Eventcode_1],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Eventcode_1); \
    }


 
/* static access macros for signal LI0_MFL_Eventcode_2 */
 
#define l_u8_rd_LI0_MFL_Eventcode_2() \
    ((l_u8)  (((lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Eventcode_2]) >> 4U) & 0x0fU))


#define l_u8_wr_LI0_MFL_Eventcode_2(A) \
    { \
    lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Eventcode_2] = \
    (l_u8)((lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Eventcode_2] & 0x0fU) | \
    (((A) << 4U) & 0xf0U)); \
    LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Eventcode_2],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Eventcode_2); \
    }


 
/* static access macros for signal LI0_MFL_Marke */
 
#define l_u8_rd_LI0_MFL_Marke() \
    ((l_u8)  (((lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Marke]) >> 0U) & 0x0fU))


#define l_u8_wr_LI0_MFL_Marke(A) \
    { \
    lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Marke] = \
    (l_u8)((lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Marke] & 0xf0U) | \
    (((A) << 0U) & 0x0fU)); \
    LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Marke],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Marke); \
    }


 
/* static access macros for signal LI0_MFL_Variante */
 
#define l_u8_rd_LI0_MFL_Variante() \
    ((l_u8)  (((lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Variante]) >> 4U) & 0x0fU))


#define l_u8_wr_LI0_MFL_Variante(A) \
    { \
    lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Variante] = \
    (l_u8)((lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Variante] & 0x0fU) | \
    (((A) << 4U) & 0xf0U)); \
    LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Variante],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Variante); \
    }


 
/* static access macros for signal LI0_MFL_Dummy_0_Signal_1 */
 
#define l_u8_rd_LI0_MFL_Dummy_0_Signal_1() \
    ((l_u8)  (((lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Dummy_0_Signal_1]) >> 0U) & 0xffU))


#define l_u8_wr_LI0_MFL_Dummy_0_Signal_1(A) \
    { \
    lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Dummy_0_Signal_1] = \
    (l_u8)((lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Dummy_0_Signal_1] & 0x00U) | \
    (((A) << 0U) & 0xffU)); \
    LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Dummy_0_Signal_1],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Dummy_0_Signal_1); \
    }


/* static access macros for signal LI0_MFL_Tip_Down */

 
#define l_bool_rd_LI0_MFL_Tip_Down() \
    (LIN_TEST_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Tip_Down], \
    LIN_BIT_OFFSET_LI0_MFL_Tip_Down))

#define l_bool_wr_LI0_MFL_Tip_Down(A) \
    {(A) ? \
    (LIN_SET_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Tip_Down], \
    LIN_BIT_OFFSET_LI0_MFL_Tip_Down)):\
    (LIN_CLEAR_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Tip_Down], \
    LIN_BIT_OFFSET_LI0_MFL_Tip_Down));\
    LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Tip_Down],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Tip_Down);}
/* static access macros for signal LI0_MFL_Tip_Up */

 
#define l_bool_rd_LI0_MFL_Tip_Up() \
    (LIN_TEST_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Tip_Up], \
    LIN_BIT_OFFSET_LI0_MFL_Tip_Up))

#define l_bool_wr_LI0_MFL_Tip_Up(A) \
    {(A) ? \
    (LIN_SET_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Tip_Up], \
    LIN_BIT_OFFSET_LI0_MFL_Tip_Up)):\
    (LIN_CLEAR_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Tip_Up], \
    LIN_BIT_OFFSET_LI0_MFL_Tip_Up));\
    LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Tip_Up],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Tip_Up);}
/* static access macros for signal LI0_MFL_SatModul_links_Err */

 
#define l_bool_rd_LI0_MFL_SatModul_links_Err() \
    (LIN_TEST_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_SatModul_links_Err], \
    LIN_BIT_OFFSET_LI0_MFL_SatModul_links_Err))

#define l_bool_wr_LI0_MFL_SatModul_links_Err(A) \
    {(A) ? \
    (LIN_SET_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_SatModul_links_Err], \
    LIN_BIT_OFFSET_LI0_MFL_SatModul_links_Err)):\
    (LIN_CLEAR_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_SatModul_links_Err], \
    LIN_BIT_OFFSET_LI0_MFL_SatModul_links_Err));\
    LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_SatModul_links_Err],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_SatModul_links_Err);}
/* static access macros for signal LI0_MFL_SatModul_rechts_Err */

 
#define l_bool_rd_LI0_MFL_SatModul_rechts_Err() \
    (LIN_TEST_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_SatModul_rechts_Err], \
    LIN_BIT_OFFSET_LI0_MFL_SatModul_rechts_Err))

#define l_bool_wr_LI0_MFL_SatModul_rechts_Err(A) \
    {(A) ? \
    (LIN_SET_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_SatModul_rechts_Err], \
    LIN_BIT_OFFSET_LI0_MFL_SatModul_rechts_Err)):\
    (LIN_CLEAR_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_SatModul_rechts_Err], \
    LIN_BIT_OFFSET_LI0_MFL_SatModul_rechts_Err));\
    LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_SatModul_rechts_Err],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_SatModul_rechts_Err);}
/* static access macros for signal LI0_MFL_Dummy_0_Signal_2 */

 
#define l_bool_rd_LI0_MFL_Dummy_0_Signal_2() \
    (LIN_TEST_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Dummy_0_Signal_2], \
    LIN_BIT_OFFSET_LI0_MFL_Dummy_0_Signal_2))

#define l_bool_wr_LI0_MFL_Dummy_0_Signal_2(A) \
    {(A) ? \
    (LIN_SET_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Dummy_0_Signal_2], \
    LIN_BIT_OFFSET_LI0_MFL_Dummy_0_Signal_2)):\
    (LIN_CLEAR_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Dummy_0_Signal_2], \
    LIN_BIT_OFFSET_LI0_MFL_Dummy_0_Signal_2));\
    LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Dummy_0_Signal_2],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Dummy_0_Signal_2);}
/* static access macros for signal LI0_MFL_Dummy_0_Signal_3 */

 
#define l_bool_rd_LI0_MFL_Dummy_0_Signal_3() \
    (LIN_TEST_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Dummy_0_Signal_3], \
    LIN_BIT_OFFSET_LI0_MFL_Dummy_0_Signal_3))

#define l_bool_wr_LI0_MFL_Dummy_0_Signal_3(A) \
    {(A) ? \
    (LIN_SET_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Dummy_0_Signal_3], \
    LIN_BIT_OFFSET_LI0_MFL_Dummy_0_Signal_3)):\
    (LIN_CLEAR_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Dummy_0_Signal_3], \
    LIN_BIT_OFFSET_LI0_MFL_Dummy_0_Signal_3));\
    LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Dummy_0_Signal_3],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Dummy_0_Signal_3);}
/* static access macros for signal LI0_MFL_LR_HZG_Status */

 
#define l_bool_rd_LI0_MFL_LR_HZG_Status() \
    (LIN_TEST_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_LR_HZG_Status], \
    LIN_BIT_OFFSET_LI0_MFL_LR_HZG_Status))

#define l_bool_wr_LI0_MFL_LR_HZG_Status(A) \
    {(A) ? \
    (LIN_SET_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_LR_HZG_Status], \
    LIN_BIT_OFFSET_LI0_MFL_LR_HZG_Status)):\
    (LIN_CLEAR_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_LR_HZG_Status], \
    LIN_BIT_OFFSET_LI0_MFL_LR_HZG_Status));\
    LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_LR_HZG_Status],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_LR_HZG_Status);}
/* static access macros for signal LI0_MFL_LR_HZG_Err */

 
#define l_bool_rd_LI0_MFL_LR_HZG_Err() \
    (LIN_TEST_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_LR_HZG_Err], \
    LIN_BIT_OFFSET_LI0_MFL_LR_HZG_Err))

#define l_bool_wr_LI0_MFL_LR_HZG_Err(A) \
    {(A) ? \
    (LIN_SET_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_LR_HZG_Err], \
    LIN_BIT_OFFSET_LI0_MFL_LR_HZG_Err)):\
    (LIN_CLEAR_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_LR_HZG_Err], \
    LIN_BIT_OFFSET_LI0_MFL_LR_HZG_Err));\
    LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_LR_HZG_Err],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_LR_HZG_Err);}
/* static access macros for signal LI0_MFL_Signalhorn */

 
#define l_bool_rd_LI0_MFL_Signalhorn() \
    (LIN_TEST_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Signalhorn], \
    LIN_BIT_OFFSET_LI0_MFL_Signalhorn))

#define l_bool_wr_LI0_MFL_Signalhorn(A) \
    {(A) ? \
    (LIN_SET_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Signalhorn], \
    LIN_BIT_OFFSET_LI0_MFL_Signalhorn)):\
    (LIN_CLEAR_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Signalhorn], \
    LIN_BIT_OFFSET_LI0_MFL_Signalhorn));\
    LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Signalhorn],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Signalhorn);}
/* static access macros for signal LI0_MFL_Signalhorn_Err */

 
#define l_bool_rd_LI0_MFL_Signalhorn_Err() \
    (LIN_TEST_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Signalhorn_Err], \
    LIN_BIT_OFFSET_LI0_MFL_Signalhorn_Err))

#define l_bool_wr_LI0_MFL_Signalhorn_Err(A) \
    {(A) ? \
    (LIN_SET_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Signalhorn_Err], \
    LIN_BIT_OFFSET_LI0_MFL_Signalhorn_Err)):\
    (LIN_CLEAR_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Signalhorn_Err], \
    LIN_BIT_OFFSET_LI0_MFL_Signalhorn_Err));\
    LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Signalhorn_Err],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Signalhorn_Err);}
/* static access macros for signal LI0_MFL_Tip_links_Err */

 
#define l_bool_rd_LI0_MFL_Tip_links_Err() \
    (LIN_TEST_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Tip_links_Err], \
    LIN_BIT_OFFSET_LI0_MFL_Tip_links_Err))

#define l_bool_wr_LI0_MFL_Tip_links_Err(A) \
    {(A) ? \
    (LIN_SET_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Tip_links_Err], \
    LIN_BIT_OFFSET_LI0_MFL_Tip_links_Err)):\
    (LIN_CLEAR_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Tip_links_Err], \
    LIN_BIT_OFFSET_LI0_MFL_Tip_links_Err));\
    LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Tip_links_Err],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Tip_links_Err);}
/* static access macros for signal LI0_MFL_Tip_rechts_Err */

 
#define l_bool_rd_LI0_MFL_Tip_rechts_Err() \
    (LIN_TEST_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Tip_rechts_Err], \
    LIN_BIT_OFFSET_LI0_MFL_Tip_rechts_Err))

#define l_bool_wr_LI0_MFL_Tip_rechts_Err(A) \
    {(A) ? \
    (LIN_SET_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Tip_rechts_Err], \
    LIN_BIT_OFFSET_LI0_MFL_Tip_rechts_Err)):\
    (LIN_CLEAR_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Tip_rechts_Err], \
    LIN_BIT_OFFSET_LI0_MFL_Tip_rechts_Err));\
    LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Tip_rechts_Err],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Tip_rechts_Err);}
/* static access macros for signal LI0_MFL_Taste_links_Err */

 
#define l_bool_rd_LI0_MFL_Taste_links_Err() \
    (LIN_TEST_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Taste_links_Err], \
    LIN_BIT_OFFSET_LI0_MFL_Taste_links_Err))

#define l_bool_wr_LI0_MFL_Taste_links_Err(A) \
    {(A) ? \
    (LIN_SET_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Taste_links_Err], \
    LIN_BIT_OFFSET_LI0_MFL_Taste_links_Err)):\
    (LIN_CLEAR_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Taste_links_Err], \
    LIN_BIT_OFFSET_LI0_MFL_Taste_links_Err));\
    LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Taste_links_Err],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Taste_links_Err);}
/* static access macros for signal LI0_MFL_Taste_rechts_Err */

 
#define l_bool_rd_LI0_MFL_Taste_rechts_Err() \
    (LIN_TEST_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Taste_rechts_Err], \
    LIN_BIT_OFFSET_LI0_MFL_Taste_rechts_Err))

#define l_bool_wr_LI0_MFL_Taste_rechts_Err(A) \
    {(A) ? \
    (LIN_SET_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Taste_rechts_Err], \
    LIN_BIT_OFFSET_LI0_MFL_Taste_rechts_Err)):\
    (LIN_CLEAR_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Taste_rechts_Err], \
    LIN_BIT_OFFSET_LI0_MFL_Taste_rechts_Err));\
    LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Taste_rechts_Err],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Taste_rechts_Err);}
/* static access macros for signal LI0_MFL_ECU_Err */

 
#define l_bool_rd_LI0_MFL_ECU_Err() \
    (LIN_TEST_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_ECU_Err], \
    LIN_BIT_OFFSET_LI0_MFL_ECU_Err))

#define l_bool_wr_LI0_MFL_ECU_Err(A) \
    {(A) ? \
    (LIN_SET_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_ECU_Err], \
    LIN_BIT_OFFSET_LI0_MFL_ECU_Err)):\
    (LIN_CLEAR_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_ECU_Err], \
    LIN_BIT_OFFSET_LI0_MFL_ECU_Err));\
    LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_ECU_Err],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_ECU_Err);}
/* static access macros for signal LI0_MFL_Response_Err */

 
#define l_bool_rd_LI0_MFL_Response_Err() \
    (LIN_TEST_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Response_Err], \
    LIN_BIT_OFFSET_LI0_MFL_Response_Err))

#define l_bool_wr_LI0_MFL_Response_Err(A) \
    {(A) ? \
    (LIN_SET_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Response_Err], \
    LIN_BIT_OFFSET_LI0_MFL_Response_Err)):\
    (LIN_CLEAR_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_MFL_Response_Err], \
    LIN_BIT_OFFSET_LI0_MFL_Response_Err));\
    LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Response_Err],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Response_Err);}
 
/* static access macros for signal LI0_DI_KL_58xs */
 
#define l_u8_rd_LI0_DI_KL_58xs() \
    ((l_u8)  (((lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_DI_KL_58xs]) >> 0U) & 0x7fU))


#define l_u8_wr_LI0_DI_KL_58xs(A) \
    { \
    lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_DI_KL_58xs] = \
    (l_u8)((lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_DI_KL_58xs] & 0x80U) | \
    (((A) << 0U) & 0x7fU)); \
    LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_DI_KL_58xs],\
         LIN_FLAG_BIT_OFFSET_LI0_DI_KL_58xs); \
    }


/* static access macros for signal LI0_DI_Emblem */

 
#define l_bool_rd_LI0_DI_Emblem() \
    (LIN_TEST_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_DI_Emblem], \
    LIN_BIT_OFFSET_LI0_DI_Emblem))

#define l_bool_wr_LI0_DI_Emblem(A) \
    {(A) ? \
    (LIN_SET_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_DI_Emblem], \
    LIN_BIT_OFFSET_LI0_DI_Emblem)):\
    (LIN_CLEAR_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_DI_Emblem], \
    LIN_BIT_OFFSET_LI0_DI_Emblem));\
    LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_DI_Emblem],\
         LIN_FLAG_BIT_OFFSET_LI0_DI_Emblem);}
/* static access macros for signal LI0_Klemme_15_MFL */

 
#define l_bool_rd_LI0_Klemme_15_MFL() \
    (LIN_TEST_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_Klemme_15_MFL], \
    LIN_BIT_OFFSET_LI0_Klemme_15_MFL))

#define l_bool_wr_LI0_Klemme_15_MFL(A) \
    {(A) ? \
    (LIN_SET_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_Klemme_15_MFL], \
    LIN_BIT_OFFSET_LI0_Klemme_15_MFL)):\
    (LIN_CLEAR_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_Klemme_15_MFL], \
    LIN_BIT_OFFSET_LI0_Klemme_15_MFL));\
    LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_Klemme_15_MFL],\
         LIN_FLAG_BIT_OFFSET_LI0_Klemme_15_MFL);}
 
/* static access macros for signal LI0_void_1 */
 
#define l_u8_rd_LI0_void_1() \
    ((l_u8)  (((lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_void_1]) >> 1U) & 0x3fU))


#define l_u8_wr_LI0_void_1(A) \
    { \
    lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_void_1] = \
    (l_u8)((lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_void_1] & 0x81U) | \
    (((A) << 1U) & 0x7eU)); \
    LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_void_1],\
         LIN_FLAG_BIT_OFFSET_LI0_void_1); \
    }


/* static access macros for signal LI0_Dimmung_01_gueltig */

 
#define l_bool_rd_LI0_Dimmung_01_gueltig() \
    (LIN_TEST_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_Dimmung_01_gueltig], \
    LIN_BIT_OFFSET_LI0_Dimmung_01_gueltig))

#define l_bool_wr_LI0_Dimmung_01_gueltig(A) \
    {(A) ? \
    (LIN_SET_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_Dimmung_01_gueltig], \
    LIN_BIT_OFFSET_LI0_Dimmung_01_gueltig)):\
    (LIN_CLEAR_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_Dimmung_01_gueltig], \
    LIN_BIT_OFFSET_LI0_Dimmung_01_gueltig));\
    LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_Dimmung_01_gueltig],\
         LIN_FLAG_BIT_OFFSET_LI0_Dimmung_01_gueltig);}
 
/* static access macros for signal LI0_DI_KL_58xt */
 
#define l_u8_rd_LI0_DI_KL_58xt() \
    ((l_u8)  (((lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_DI_KL_58xt]) >> 0U) & 0x7fU))


#define l_u8_wr_LI0_DI_KL_58xt(A) \
    { \
    lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_DI_KL_58xt] = \
    (l_u8)((lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_DI_KL_58xt] & 0x80U) | \
    (((A) << 0U) & 0x7fU)); \
    LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_DI_KL_58xt],\
         LIN_FLAG_BIT_OFFSET_LI0_DI_KL_58xt); \
    }


/* static access macros for signal LI0_void_2 */

 
#define l_bool_rd_LI0_void_2() \
    (LIN_TEST_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_void_2], \
    LIN_BIT_OFFSET_LI0_void_2))

#define l_bool_wr_LI0_void_2(A) \
    {(A) ? \
    (LIN_SET_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_void_2], \
    LIN_BIT_OFFSET_LI0_void_2)):\
    (LIN_CLEAR_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_void_2], \
    LIN_BIT_OFFSET_LI0_void_2));\
    LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_void_2],\
         LIN_FLAG_BIT_OFFSET_LI0_void_2);}
 
/* static access macros for signal LI0_void_3 */
 
#define l_u8_rd_LI0_void_3() \
    ((l_u8)  (((lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_void_3]) >> 0U) & 0x7fU))


#define l_u8_wr_LI0_void_3(A) \
    { \
    lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_void_3] = \
    (l_u8)((lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_void_3] & 0x80U) | \
    (((A) << 0U) & 0x7fU)); \
    LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_void_3],\
         LIN_FLAG_BIT_OFFSET_LI0_void_3); \
    }


/* static access macros for signal LI0_P_Mode_MFL */

 
#define l_bool_rd_LI0_P_Mode_MFL() \
    (LIN_TEST_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_P_Mode_MFL], \
    LIN_BIT_OFFSET_LI0_P_Mode_MFL))

#define l_bool_wr_LI0_P_Mode_MFL(A) \
    {(A) ? \
    (LIN_SET_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_P_Mode_MFL], \
    LIN_BIT_OFFSET_LI0_P_Mode_MFL)):\
    (LIN_CLEAR_BIT(lin_pFrameBuf[LIN_BYTE_OFFSET_LI0_P_Mode_MFL], \
    LIN_BIT_OFFSET_LI0_P_Mode_MFL));\
    LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_P_Mode_MFL],\
         LIN_FLAG_BIT_OFFSET_LI0_P_Mode_MFL);}


/* Signal flag APIs */

#define l_flg_tst_LI0_MFL_Zaehler_flag() \
         LIN_TEST_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Zaehler],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Zaehler)
#define l_flg_clr_LI0_MFL_Zaehler_flag() \
         LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Zaehler],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Zaehler)

#define l_flg_tst_LI0_MFL_Toggle_flag() \
         LIN_TEST_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Toggle],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Toggle)
#define l_flg_clr_LI0_MFL_Toggle_flag() \
         LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Toggle],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Toggle)

#define l_flg_tst_LI0_MFL_Lokalaktiv_flag() \
         LIN_TEST_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Lokalaktiv],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Lokalaktiv)
#define l_flg_clr_LI0_MFL_Lokalaktiv_flag() \
         LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Lokalaktiv],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Lokalaktiv)

#define l_flg_tst_LI0_MFL_M_Taste_flag() \
         LIN_TEST_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_M_Taste],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_M_Taste)
#define l_flg_clr_LI0_MFL_M_Taste_flag() \
         LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_M_Taste],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_M_Taste)

#define l_flg_tst_LI0_void_flag() \
         LIN_TEST_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_void],\
         LIN_FLAG_BIT_OFFSET_LI0_void)
#define l_flg_clr_LI0_void_flag() \
         LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_void],\
         LIN_FLAG_BIT_OFFSET_LI0_void)

#define l_flg_tst_LI0_MFL_Tastencode_1_flag() \
         LIN_TEST_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Tastencode_1],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Tastencode_1)
#define l_flg_clr_LI0_MFL_Tastencode_1_flag() \
         LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Tastencode_1],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Tastencode_1)

#define l_flg_tst_LI0_MFL_Tastencode_2_flag() \
         LIN_TEST_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Tastencode_2],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Tastencode_2)
#define l_flg_clr_LI0_MFL_Tastencode_2_flag() \
         LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Tastencode_2],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Tastencode_2)

#define l_flg_tst_LI0_MFL_Eventcode_1_flag() \
         LIN_TEST_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Eventcode_1],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Eventcode_1)
#define l_flg_clr_LI0_MFL_Eventcode_1_flag() \
         LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Eventcode_1],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Eventcode_1)

#define l_flg_tst_LI0_MFL_Eventcode_2_flag() \
         LIN_TEST_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Eventcode_2],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Eventcode_2)
#define l_flg_clr_LI0_MFL_Eventcode_2_flag() \
         LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Eventcode_2],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Eventcode_2)

#define l_flg_tst_LI0_MFL_Marke_flag() \
         LIN_TEST_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Marke],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Marke)
#define l_flg_clr_LI0_MFL_Marke_flag() \
         LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Marke],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Marke)

#define l_flg_tst_LI0_MFL_Variante_flag() \
         LIN_TEST_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Variante],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Variante)
#define l_flg_clr_LI0_MFL_Variante_flag() \
         LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Variante],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Variante)

#define l_flg_tst_LI0_MFL_Dummy_0_Signal_1_flag() \
         LIN_TEST_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Dummy_0_Signal_1],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Dummy_0_Signal_1)
#define l_flg_clr_LI0_MFL_Dummy_0_Signal_1_flag() \
         LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Dummy_0_Signal_1],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Dummy_0_Signal_1)

#define l_flg_tst_LI0_MFL_Tip_Down_flag() \
         LIN_TEST_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Tip_Down],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Tip_Down)
#define l_flg_clr_LI0_MFL_Tip_Down_flag() \
         LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Tip_Down],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Tip_Down)

#define l_flg_tst_LI0_MFL_Tip_Up_flag() \
         LIN_TEST_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Tip_Up],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Tip_Up)
#define l_flg_clr_LI0_MFL_Tip_Up_flag() \
         LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Tip_Up],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Tip_Up)

#define l_flg_tst_LI0_MFL_SatModul_links_Err_flag() \
         LIN_TEST_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_SatModul_links_Err],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_SatModul_links_Err)
#define l_flg_clr_LI0_MFL_SatModul_links_Err_flag() \
         LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_SatModul_links_Err],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_SatModul_links_Err)

#define l_flg_tst_LI0_MFL_SatModul_rechts_Err_flag() \
         LIN_TEST_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_SatModul_rechts_Err],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_SatModul_rechts_Err)
#define l_flg_clr_LI0_MFL_SatModul_rechts_Err_flag() \
         LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_SatModul_rechts_Err],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_SatModul_rechts_Err)

#define l_flg_tst_LI0_MFL_Dummy_0_Signal_2_flag() \
         LIN_TEST_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Dummy_0_Signal_2],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Dummy_0_Signal_2)
#define l_flg_clr_LI0_MFL_Dummy_0_Signal_2_flag() \
         LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Dummy_0_Signal_2],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Dummy_0_Signal_2)

#define l_flg_tst_LI0_MFL_Dummy_0_Signal_3_flag() \
         LIN_TEST_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Dummy_0_Signal_3],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Dummy_0_Signal_3)
#define l_flg_clr_LI0_MFL_Dummy_0_Signal_3_flag() \
         LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Dummy_0_Signal_3],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Dummy_0_Signal_3)

#define l_flg_tst_LI0_MFL_LR_HZG_Status_flag() \
         LIN_TEST_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_LR_HZG_Status],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_LR_HZG_Status)
#define l_flg_clr_LI0_MFL_LR_HZG_Status_flag() \
         LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_LR_HZG_Status],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_LR_HZG_Status)

#define l_flg_tst_LI0_MFL_LR_HZG_Err_flag() \
         LIN_TEST_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_LR_HZG_Err],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_LR_HZG_Err)
#define l_flg_clr_LI0_MFL_LR_HZG_Err_flag() \
         LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_LR_HZG_Err],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_LR_HZG_Err)

#define l_flg_tst_LI0_MFL_Signalhorn_flag() \
         LIN_TEST_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Signalhorn],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Signalhorn)
#define l_flg_clr_LI0_MFL_Signalhorn_flag() \
         LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Signalhorn],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Signalhorn)

#define l_flg_tst_LI0_MFL_Signalhorn_Err_flag() \
         LIN_TEST_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Signalhorn_Err],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Signalhorn_Err)
#define l_flg_clr_LI0_MFL_Signalhorn_Err_flag() \
         LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Signalhorn_Err],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Signalhorn_Err)

#define l_flg_tst_LI0_MFL_Tip_links_Err_flag() \
         LIN_TEST_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Tip_links_Err],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Tip_links_Err)
#define l_flg_clr_LI0_MFL_Tip_links_Err_flag() \
         LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Tip_links_Err],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Tip_links_Err)

#define l_flg_tst_LI0_MFL_Tip_rechts_Err_flag() \
         LIN_TEST_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Tip_rechts_Err],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Tip_rechts_Err)
#define l_flg_clr_LI0_MFL_Tip_rechts_Err_flag() \
         LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Tip_rechts_Err],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Tip_rechts_Err)

#define l_flg_tst_LI0_MFL_Taste_links_Err_flag() \
         LIN_TEST_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Taste_links_Err],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Taste_links_Err)
#define l_flg_clr_LI0_MFL_Taste_links_Err_flag() \
         LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Taste_links_Err],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Taste_links_Err)

#define l_flg_tst_LI0_MFL_Taste_rechts_Err_flag() \
         LIN_TEST_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Taste_rechts_Err],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Taste_rechts_Err)
#define l_flg_clr_LI0_MFL_Taste_rechts_Err_flag() \
         LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Taste_rechts_Err],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Taste_rechts_Err)

#define l_flg_tst_LI0_MFL_ECU_Err_flag() \
         LIN_TEST_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_ECU_Err],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_ECU_Err)
#define l_flg_clr_LI0_MFL_ECU_Err_flag() \
         LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_ECU_Err],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_ECU_Err)

#define l_flg_tst_LI0_MFL_Response_Err_flag() \
         LIN_TEST_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Response_Err],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Response_Err)
#define l_flg_clr_LI0_MFL_Response_Err_flag() \
         LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_MFL_Response_Err],\
         LIN_FLAG_BIT_OFFSET_LI0_MFL_Response_Err)

#define l_flg_tst_LI0_DI_KL_58xs_flag() \
         LIN_TEST_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_DI_KL_58xs],\
         LIN_FLAG_BIT_OFFSET_LI0_DI_KL_58xs)
#define l_flg_clr_LI0_DI_KL_58xs_flag() \
         LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_DI_KL_58xs],\
         LIN_FLAG_BIT_OFFSET_LI0_DI_KL_58xs)

#define l_flg_tst_LI0_DI_Emblem_flag() \
         LIN_TEST_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_DI_Emblem],\
         LIN_FLAG_BIT_OFFSET_LI0_DI_Emblem)
#define l_flg_clr_LI0_DI_Emblem_flag() \
         LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_DI_Emblem],\
         LIN_FLAG_BIT_OFFSET_LI0_DI_Emblem)

#define l_flg_tst_LI0_Klemme_15_MFL_flag() \
         LIN_TEST_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_Klemme_15_MFL],\
         LIN_FLAG_BIT_OFFSET_LI0_Klemme_15_MFL)
#define l_flg_clr_LI0_Klemme_15_MFL_flag() \
         LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_Klemme_15_MFL],\
         LIN_FLAG_BIT_OFFSET_LI0_Klemme_15_MFL)

#define l_flg_tst_LI0_void_1_flag() \
         LIN_TEST_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_void_1],\
         LIN_FLAG_BIT_OFFSET_LI0_void_1)
#define l_flg_clr_LI0_void_1_flag() \
         LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_void_1],\
         LIN_FLAG_BIT_OFFSET_LI0_void_1)

#define l_flg_tst_LI0_Dimmung_01_gueltig_flag() \
         LIN_TEST_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_Dimmung_01_gueltig],\
         LIN_FLAG_BIT_OFFSET_LI0_Dimmung_01_gueltig)
#define l_flg_clr_LI0_Dimmung_01_gueltig_flag() \
         LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_Dimmung_01_gueltig],\
         LIN_FLAG_BIT_OFFSET_LI0_Dimmung_01_gueltig)

#define l_flg_tst_LI0_DI_KL_58xt_flag() \
         LIN_TEST_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_DI_KL_58xt],\
         LIN_FLAG_BIT_OFFSET_LI0_DI_KL_58xt)
#define l_flg_clr_LI0_DI_KL_58xt_flag() \
         LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_DI_KL_58xt],\
         LIN_FLAG_BIT_OFFSET_LI0_DI_KL_58xt)

#define l_flg_tst_LI0_void_2_flag() \
         LIN_TEST_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_void_2],\
         LIN_FLAG_BIT_OFFSET_LI0_void_2)
#define l_flg_clr_LI0_void_2_flag() \
         LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_void_2],\
         LIN_FLAG_BIT_OFFSET_LI0_void_2)

#define l_flg_tst_LI0_void_3_flag() \
         LIN_TEST_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_void_3],\
         LIN_FLAG_BIT_OFFSET_LI0_void_3)
#define l_flg_clr_LI0_void_3_flag() \
         LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_void_3],\
         LIN_FLAG_BIT_OFFSET_LI0_void_3)

#define l_flg_tst_LI0_P_Mode_MFL_flag() \
         LIN_TEST_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_P_Mode_MFL],\
         LIN_FLAG_BIT_OFFSET_LI0_P_Mode_MFL)
#define l_flg_clr_LI0_P_Mode_MFL_flag() \
         LIN_CLEAR_BIT(lin_flag_handle_tbl[LIN_FLAG_BYTE_OFFSET_LI0_P_Mode_MFL],\
         LIN_FLAG_BIT_OFFSET_LI0_P_Mode_MFL)



/* Frame flag APIs */

   /* Interface_name = LI0 */

 #define l_flg_tst_LI0_MFLs_01_flag() \
          lin_frame_flag_tbl[LI0_MFLs_01]
 #define l_flg_clr_LI0_MFLs_01_flag() \
          lin_frame_flag_tbl[LI0_MFLs_01] = 0

 #define l_flg_tst_LI0_MFLe_01_flag() \
          lin_frame_flag_tbl[LI0_MFLe_01]
 #define l_flg_clr_LI0_MFLe_01_flag() \
          lin_frame_flag_tbl[LI0_MFLe_01] = 0

 #define l_flg_tst_LI0_MasterReq_flag() \
          lin_frame_flag_tbl[LI0_MasterReq]
 #define l_flg_clr_LI0_MasterReq_flag() \
          lin_frame_flag_tbl[LI0_MasterReq] = 0

 #define l_flg_tst_LI0_SlaveResp_flag() \
          lin_frame_flag_tbl[LI0_SlaveResp]
 #define l_flg_clr_LI0_SlaveResp_flag() \
          lin_frame_flag_tbl[LI0_SlaveResp] = 0



/* INTERFACE MANAGEMENT */

#define l_ifc_init_LI0() l_ifc_init(LI0)



#define l_ifc_wake_up_LI0() l_ifc_wake_up(LI0)



#define l_ifc_rx_LI0() l_ifc_rx(LI0)



#define l_ifc_tx_LI0() l_ifc_tx(LI0)



#define l_ifc_aux_LI0() l_ifc_aux(LI0)



#define l_ifc_read_status_LI0() l_ifc_read_status(LI0)


#endif    /* _LIN_CFG_H_ */